% dfmux_scope: Plot time-stream and spectral data from DFMUX 0
%
% USAGE:
%
% 1. Turn on the mezzanine power (use the web interface)
%
% 2. Instruct the board to start streaming UDP samples from DFMUX 0.
%    On a Linux host (or Cygwin on Windows), type:
%
%    curl -v 'http://192.168.0.51:8080/input_daq?device_id=0&protocol=udp_socket&value=1' &
%
% 3. Send an "OK, start streaming" to the board (will cause the 'curl' job
%    running in the background from step (2) to terminate)
%
%    echo "Go" | nc -u 192.168.0.51 9000 > upd_stream.dat 
%
%    ...where 192.168.0.51 is replaced by the IP address of your board.
%
% 4. Run this program to parse upd_stream.dat and plot the results.

% data_length: the total number of samples parsed (from ALL channels), not
% necessarily the number of useful samples on a particular channel.
% Too few of these, and the spectral resolution isn't that great.
% Too many, and you wait forever for stale data to flush.

%data_length = 131072; % Use for tuning; short turnaround
%data_length = 1048576; % Moderately detailed spectral content
data_length = 4194304; % Excellent spectral detail; slow!

% How long do we wait (in seconds) between iterations?
delay = 1;

filename = 'z:/upd_stream.dat';

% Which channels do we extract?
channels = [ 0 1 3 4 ];

% Which FIR stage are we reading?
stage = 5;

% If you want to put the plots on a common scale,
% uncomment the appropriate one here.
psd_scale = [ 40 150 ];
%signal_scale = [ -2048 2048 ];

% Chances are pretty good you shouldn't change these.
dmfd_clock = 25000000;
dmfd_cic_decimation = 2048;

if(~exist(filename, 'file'))
    error('Data readout file does not exist!');
end

so = [];
ro = [];

%while(1)

	f = fopen(filename, 'rb', 'ieee-le');

    c = [];
    d = [];

    fseek(f, -data_length, 'eof');
    A = fread(f, Inf, 'uint32');

    % Extract channel index (c) and data (d) from the results
    d = bitshift(A, 4, 32);
    c = bitshift(A, -28);

    % Convert negative results from twos-complement
    negs = find(bitshift(d,-31));
    d(negs) = -bitcmp(d(negs),32);

    fclose(f);

    for n=1:length(channels)
        channel = channels(n);
        subfig = find(channels == channel);

        ci = find(c==channel);
        sfigure(1);
        subplot(length(channels),1,subfig);
        plot(d(ci));
        xlabel(sprintf('Channel %i data', channel+1));
        if(exist('signal_scale','var'))
            ylim(signal_scale);
        end

        sfigure(2);
        subplot(length(channels),1,subfig);
        periodogram(d(ci),[],2^ceil(log2(length(ci))),dmfd_clock/dmfd_cic_decimation/(2^stage));
        xlabel(sprintf('Channel %i PSD estimate', channel+1));
        if(exist('psd_scale','var'))
            ylim(psd_scale);
        end

        pause(delay);
        
        %%% GMS HOLO START
        % Orthoganality for channels (0, 1) and (2,3)
        
        % Index an equal number of samples from channels 1 and 4
        c1i = find(c==0);
        c2i = find(c==1);
        c4i = find(c==3);
        c5i = find(c==4);

        ortho_length = min([length(c1i),length(c2i), length(c4i), length(c5i)]);

        c1i = c1i(end+1-ortho_length:end);
        c2i = c2i(end+1-ortho_length:end);
        c4i = c4i(end+1-ortho_length:end);
        c5i = c5i(end+1-ortho_length:end);
        
        % Grab data
        d1 = d(c1i);
        d2 = d(c2i);
        d4 = d(c4i);
        d5 = d(c5i);
        
        % Orthoganality of each I,Q pair
        if(length(so) > 100)
            so = so(end-99:100);
            ro = ro(end-99:100);
        end
        so = [ so d1' * d2 ];
        ro = [ ro d4' * d5 ];
        
        sfigure(3);
        subplot(2,1,1);semilogy([1:length(so)],so, [1:length(so)],-so);xlabel('Signal (channels 1,2)');
        subplot(2,1,2);semilogy([1:length(ro)],ro, [1:length(so)],-ro);xlabel('Reference (channels 4,5)');
        
        % Down-mix
        dm = ( d1 + i*d2 ) .* ( d4 - i*d5 );

        sfigure(4);periodogram(dm,[],2^ceil(log2(length(dm))),dmfd_clock/dmfd_cic_decimation/(2^stage));
        title(sprintf('Down-mixed PSD estimate'));
        %%% GMS HOLO END
    end
%end